use std::io::{Read, Write};

use miette::{Context, IntoDiagnostic};
use sercon_base::Backend;
use serde_value::Value;

#[derive(Default)]
pub struct CborBackend;

impl Backend for CborBackend {
    fn serialize(&self, out: &mut dyn Write, data: Value) -> miette::Result<()> {
        ciborium::ser::into_writer(&data, out)
            .into_diagnostic()
            .wrap_err("CBOR serialization error")
    }

    fn deserialize(&self, data: &mut dyn Read) -> miette::Result<Value> {
        ciborium::de::from_reader(data)
            .into_diagnostic()
            .wrap_err("CBOR deserialization error")
    }
}

impl_filetype_backend!(CborBackend, ["cbor"]);
