use std::io::{Read, Write};

use miette::{Context, IntoDiagnostic};
use sercon_base::Backend;
use serde_value::Value;

#[derive(Default)]
pub struct JsonBackend;

impl Backend for JsonBackend {
    fn serialize(&self, out: &mut dyn Write, data: Value) -> miette::Result<()> {
        serde_json::to_writer_pretty(out, &data)
            .into_diagnostic()
            .wrap_err("JSON serialization error")
    }

    fn deserialize(&self, data: &mut dyn Read) -> miette::Result<Value> {
        serde_json::from_reader(data)
            .into_diagnostic()
            .wrap_err("JSON deserialization error")
    }
}

impl_filetype_backend!(JsonBackend, ["json"]);
