use std::io::{Read, Write};

use miette::{Context, IntoDiagnostic};
use sercon_base::Backend;
use serde_value::Value;

#[derive(Default)]
pub struct XmlBackend;

impl Backend for XmlBackend {
    fn serialize(&self, out: &mut dyn Write, data: Value) -> miette::Result<()> {
        serde_xml_rs::to_writer(out, &data)
            .into_diagnostic()
            .wrap_err("XML serialization error")
    }

    fn deserialize(&self, data: &mut dyn Read) -> miette::Result<Value> {
        serde_xml_rs::from_reader(data)
            .into_diagnostic()
            .wrap_err("XML deserialization error")
    }
}

impl_filetype_backend!(XmlBackend, ["xml", "html"]);
