use std::io::{Read, Write};

use miette::{Context, IntoDiagnostic};
use sercon_base::Backend;
use serde_value::Value;

#[derive(Default)]
pub struct YamlBackend;

impl Backend for YamlBackend {
    fn serialize(&self, out: &mut dyn Write, data: Value) -> miette::Result<()> {
        serde_yaml::to_writer(out, &data)
            .into_diagnostic()
            .wrap_err("YAML serialization error")
    }

    fn deserialize(&self, data: &mut dyn Read) -> miette::Result<Value> {
        serde_yaml::from_reader(data)
            .into_diagnostic()
            .wrap_err("YAML deserialization error")
    }
}

impl_filetype_backend!(YamlBackend, ["yml", "yaml"]);
